class Distribution:
    """Base class for distribution function classes.

    Use derived class after importing this class,
    in order to read data files.

    """

    def __init__(self, mu=0, sigma=1):
        """ Generic distribution class for calculating and
        visualizing a probability distribution.

        Attributes
        ----------
        mean: float, default=0
            representing the mean value of the distribution

        stdev: float, default=1
            representing the standard deviation of the distribution

        data: list of floats
            a list of floats to be extracted from the data file
        """

        self.mean = mu
        self.stdev = sigma
        self.data = []

    def read_data_file(self, file_name):
        """Function to read in data from a txt file.
        The txt file should have one number (float) per line.
        The numbers are stored in the data attribute.

        Parameters
        ----------
        file_name: string
            name of a file to read from
        """
        with open(file_name) as file:
            data_list = []
            line = file.readline()
            while line:
                data_list.append(int(line))
                line = file.readline()
        file.close()

        self.data = data_list



from .generaldistr import Distribution
import math
import matplotlib.pyplot as plt


class Binomial(Distribution):
    """ Binomial distribution class for calculating and
    visualizing a Binomial distribution.

    Attributes
    ----------
    mean: float, default=0
        representing the mean value of the distribution

    stdev: float, default=1
        representing the standard deviation of the distribution

    data: list of floats
        a list of floats to be extracted from the data file

    p: float, default=0.5
        representing the probability of an event occurring

    n: float, default=20
        representing the size of data set
    """
    def __init__(self, prob=0.5, size=20):

        self.p = prob
        self.n = size

        Distribution.__init__(
            self,
            self.calculate_mean(),
            self.calculate_stdev()
        )

    def calculate_mean(self):
        """Function to calculate the mean from p and n

        Returns
        -------
        self.mean: float
            mean of the data set
        """
        self.mean = self.p * self.n

        return self.mean

    def calculate_stdev(self):
        """Function to calculate the standard deviation from p and n.

        Returns
        -------
        self.stdev: float
            standard deviation of the data set
        """
        self.stdev = math.sqrt(self.n * self.p * (1 - self.p))

        return self.stdev

    def replace_stats_with_data(self):
        """Function to calculate p and n from the data set.
        The function updates the p and n variables of the object.

        Returns
        -------
        self.p: float
            the p value
        self.n: float
            the n value
        """
        self.n = len(self.data)
        self.p = 1.0 * sum(self.data)/len(self.data)
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()

        return self.p, self.n

    def plot_bar(self):
        """Function to output a histogram of the
        instance variable data using matplotlib pyplot library.
        """
        plt.bar(x = ['0', '1'], height = [(1 - self.p) * self.n,self.p * self.n])
        plt.title('Bar Chart of Data')
        plt.xlabel('outcome')
        plt.ylabel('frequency')

    def pdf(self, k):
        """Probability density function calculator
        for the binomial distribution.

        Parameters
        -------
        k: float
            point for calculating the probability
            density function.

        Returns
        -------
        float, probability density function output
        """
        first = (math.factorial(self.n) / (math.factorial(k) * math.factorial(self.n-k)))
        second = (self.p**k) * (1 - self.p)**(self.n - k)

        return first * second

    def plot_binomial_pdf(self):
        """Function to plot the pdf of the binomial distribution.

        Returns
        -------
        float, probability density function output

        Returns:
        x: list
            x values for the pdf plot
        y: list
            y values for the pdf plot
        """
        x = []
        y = []

        for i in range(self.n+1):
            x.append(i)
            y.append(self.pdf(i))

        plt.bar(x, y)
        plt.title('Distribution of Outcomes')
        plt.xlabel('probability')
        plt.ylabel('outcome')
        plt.show()

        return x, y

    def __add__(self, other):
        """Function to add together two Binomial
        distributions with equal p.

        Parameters
        -------
        other: 2nd Binomial instance

        Returns
        -------
        result: 3rd Binomial instance
            sum of two Binomial distribution instances
        """
        try:
            assert self.p == other.p, 'p values are not equal'
        except AssertionError as error:
            raise

        result = Binomial()
        result.p = self.p
        result.n = self.n + other.n
        result.calculate_mean()
        result.calculate_stdev()

        return result

    def __repr__(self):
        """Function to output the characteristics
        of the Binomial instance.

        Returns
        -------
        string, Gaussian object characteristics
        """
        return "mean {}, standard deviation {}, p {}, n {}".format(self.mean, self.stdev, self.p, self.n)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='problystatpy',
    version='0.8.3',
    author="Jenny J. Jung",
    author_email='jjung@gmail.com',
    description='Statististical Probability Distributions',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jungsnn/problystatPyP",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
)



import math
import matplotlib.pyplot as plt
from .generaldistr import Distribution


class Gaussian(Distribution):
    """ Gaussian distribution class for calculating and
    visualizing a Gaussian distribution.

    Attributes
    ----------
    mean: float, default=0
        representing the mean value of the distribution

    stdev: float, default=1
        representing the standard deviation of the distribution

    data: list of floats
        a list of floats to be extracted from the data file
    """
    def __init__(self, mu=0, sigma=1):

        Distribution.__init__(self, mu, sigma)

    def calculate_mean(self):
        """Method to calculate the mean of the data set.

        Returns
        -------
        self.mean: float
            mean of the data set
        """
        avg = 1.0 * sum(self.data) / len(self.data)

        self.mean = avg

        return self.mean

    def calculate_stdev(self, sample=True):
        """Method to calculate the standard deviation
        of the data set.

        Parameters
        -------
        sample: bool, default=True
            whether the data represents a sample or population

        Returns
        -------
        self.stdev: float
            standard deviation of the data set
        """
        if sample:
            n = len(self.data) - 1
        else:
            n = len(self.data)

        mean = self.calculate_mean()

        sigma = 0

        for d in self.data:
            sigma += (d - mean) ** 2

        sigma = math.sqrt(sigma / n)

        self.stdev = sigma

        return self.stdev

    def plot_histogram(self):
        """Method to output a histogram of the instance variable data using
        matplotlib pyplot library.
        """
        plt.hist(self.data)
        plt.title("Histogram of Data",
                  fontweight ="bold")
        plt.xlabel("data")
        plt.ylabel("count")

    def pdf(self, x):
        """Probability density function calculator
        for the gaussian distribution.

        Parameters
        -------
        x: float
            point for calculating the probability density function

        Returns
        -------
        float, probability density function output
        """
        return (1.0 / (self.stdev * math.sqrt(2*math.pi))) * math.exp(-0.5*((x - self.mean) / self.stdev) ** 2)

    def plot_histogram_pdf(self, n_spaces = 50):
        """Method to plot the normalized histogram of the
        data and a plot of the probability density function
        along the same range.

        Parameters
        -------
        n_spaces: int, default=50
            number of data points

        Returns
        -------
        x: list
            x values for the pdf plot
        y: list
            y values for the pdf plot
        """
        mu = self.mean
        sigma = self.stdev

        min_range = min(self.data)
        max_range = max(self.data)

        interval = 1.0 * (max_range - min_range) / n_spaces

        x = []
        y = []

        for i in range(n_spaces):
            tmp = min_range + interval*i
            x.append(tmp)
            y.append(self.pdf(tmp))

        fig, axes = plt.subplots(2,sharex=True)
        fig.subplots_adjust(hspace=.5)
        axes[0].hist(self.data, density=True)
        axes[0].set_title('Normed Histogram of Data')
        axes[0].set_ylabel('Density')

        axes[1].plot(x, y)
        axes[1].set_title('Normal Distribution for \n Sample Mean and Sample Standard Deviation')
        axes[0].set_ylabel('Density')
        plt.show()

        return x, y

    def __add__(self, other):
        """Magic method to add together two Gaussian distributions.

        Parameters
        -------
        other: 2nd Gaussian instance

        Returns
        -------
        result: 3rd Gaussian instance
            sum of two Gaussian distribution instances
        """
        result = Gaussian()

        result.mean = self.mean + other.mean
        result.stdev = math.sqrt(self.stdev**2 + other.stdev**2)

        return result

    def __repr__(self):
        """Magic method to output the characteristics
        of the Gaussian instance.

        Returns
        -------
        string, Gaussian object characteristics
        """
        return "mean {}, standard deviation {}".format(self.mean, self.stdev)
